;
; Checks the documentation for various errors

#Region Includes
#include "Array.au3"
#include "FileConstants.au3"
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProject = "Check Include libfunctions .txt"
Global Const $g_sProjectDir = "docs\autoit\english\txt2htm\txtlibfunctions"

#EndRegion Global Variables

#Region Main body of code
Global $g_nExitCode = Main($g_sProject)
Exit $g_nExitCode
#EndRegion Main body of code

#Region Main()
; ===================================================================
; Main()
;
; The main program body.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func Main($sProject)
	Local $nReturn = 0

	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $sProject & ") ====" & @CRLF)

	; Set the build directory based on the rules and the INI file value.
	_SettingBuildDir()

	; Get Functions from Include Files.
	Local $aFunctions = Get_FunctionsInclude("install\include")

	; Check #include in .txt files.
	$nReturn = Check_includeTxt($g_sProjectDir, $aFunctions)
	; Check the return value.
	If $nReturn Then
		Update_CategorieToc($g_sProjectDir, $aFunctions)
		Update_TxtInclude($g_sProjectDir, $aFunctions)
		_OutputProgressWrite("Checking failed (" & $nReturn & ")." & @CRLF)
	Else
		_OutputProgressWrite("Checking complete." & @CRLF)
	EndIf
	ToolTip("")

	; Write closing message and wait for close (if applicable).
	_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF's in case of chained output.
	_OutputWaitClosed($nReturn)

	; Return the value.
	Return $nReturn
EndFunc   ;==>Main
#EndRegion Main()

#Region Primary Functions

#Region Get_FunctionsInclude()

Func Get_FunctionsInclude($sIncludeDir)
	Local $sFile, $sIncludeFile, $aCurrentFunctions, $aFunctions[1][2], $nFunctions = 0
	Local $hSearch = FileFindFirstFile($sIncludeDir & "\*.au3")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop
		If StringInStr($sFile, "Constants") Then ContinueLoop

		$sIncludeFile = FileRead($sIncludeDir & "\" & $sFile)
		$aCurrentFunctions = GetCurrentFunctions($sIncludeFile, $sFile)
		If IsArray($aCurrentFunctions) Then
			ReDim $aFunctions[$nFunctions + UBound($aCurrentFunctions)][2]
			For $i = 0 To UBound($aCurrentFunctions) - 1
				$aFunctions[$nFunctions][0] = $aCurrentFunctions[$i][0]
				$aFunctions[$nFunctions][1] = $aCurrentFunctions[$i][1]
				$nFunctions += 1
			Next
		Else
;~ 			_OutputBuildWrite("No CURRENT section in " & $sFile & @CRLF)
		EndIf
	WEnd

	; Close the search handle
	FileClose($hSearch)

	Return $aFunctions

EndFunc   ;==>Get_FunctionsInclude

#EndRegion Get_FunctionsInclude()

#Region Check_includeTxt()

Func Check_includeTxt($sProjectDir, ByRef $aFunctions)
	_OutputProgressWrite("LibFunctions Include verification ..." & @CRLF)
	Local $nReturn = 0
	Local $sFile, $sTxtFile
	Local $hSearch = FileFindFirstFile($sProjectDir & "\*.txt")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop
		If StringInStr($sFile, "$tag") Then ContinueLoop
		ToolTip($sFile, 200, 200)

		$sTxtFile = FileReadLine($sProjectDir & "\" & $sFile, 8)
		If Not Verify_include($sTxtFile, $aFunctions, $sFile) Then
			$nReturn += 1
			_OutputBuildWrite($nReturn & " " & $sTxtFile & " wrong Include line in " & $sFile & @CRLF)
		Else
;~ 			_OutputBuildWrite($sTxtFile & " OK Include in " & $sFile & @CRLF)
		EndIf
	WEnd

	; Close the search handle
	FileClose($hSearch)

	If $nReturn Then _OutputBuildWrite(@CRLF)
	Return $nReturn
EndFunc   ;==>Check_includeTxt

#EndRegion Check_includeTxt()

#Region Update_CategorieToc()

Func Update_CategorieToc($sProjectDir, ByRef $aFunctions)
	_OutputBuildWrite("Categories.toc Updating ..." & @CRLF)
	Local $sIncludeName, $aToc = FileReadToArray($sProjectDir & "\Categories.toc")
	For $i = 0 To UBound($aToc) - 1
		If StringLeft($aToc[$i], 6) <> "WinAPI" Then ContinueLoop
		For $j = 0 To UBound($aFunctions) - 1
			If $aFunctions[$j][1] And StringInStr($aToc[$i], $aFunctions[$j][0]) Then
				$sIncludeName = StringTrimRight($aFunctions[$j][1], 4)
				If StringInStr($aToc[$i], "|_") Then
					If $sIncludeName <> "WinAPI" Then
						$aToc[$i] = StringReplace($aToc[$i], "|_", "|" & $sIncludeName & ":_")
						$aToc[$i] = StringReplace($aToc[$i], "WinAPI ", "WinAPIEx ")
					EndIf
				ElseIf StringInStr($aToc[$i], ":_") Then
					$aToc[$i] = StringRegExpReplace($aToc[$i], "\|(\w*?):", "|" & $sIncludeName & ":")
				EndIf
				ExitLoop
			EndIf
		Next
	Next
	Local $hToc = FileOpen($sProjectDir & "\Categories.toc", $FO_OVERWRITE)
	For $i = 0 To UBound($aToc) - 1
		FileWriteLine($hToc, $aToc[$i])
	Next
	FileClose($hToc)
EndFunc   ;==>Update_CategorieToc

#EndRegion Update_CategorieToc()

#Region Update_TxtInclude()

Func Update_TxtInclude($sProjectDir, ByRef $aFunctions)
	_OutputBuildWrite("^.txt include Updating ..." & @CRLF)
	Local $aTxt, $sFileName, $sIncludeName, $hToc
	For $i = 0 To UBound($aFunctions) - 1
		$sFileName = $sProjectDir & "\" & $aFunctions[$i][0] & ".txt"
		$aTxt = FileReadToArray($sFileName)
		If @error And $aFunctions[$i][1] Then
			ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : $sFileName = ' & $sFileName & @CRLF & '>Error code: ' & @error & '    Extended code: ' & @extended & ' (0x' & Hex(@extended) & ')' & @CRLF) ;### Debug Console
		EndIf

		If $aFunctions[$i][1] And StringInStr($aTxt[7], "#include") Then
			$sIncludeName = $aFunctions[$i][1]

			$aTxt[7] = StringRegExpReplace($aTxt[7], "<(.*?)>", "<" & $sIncludeName & ">")
			If @extended Then
				$hToc = FileOpen($sFileName, $FO_OVERWRITE)
				For $j = 0 To UBound($aTxt) - 1
					FileWriteLine($hToc, $aTxt[$j])
				Next
				FileClose($hToc)
			Else
				_OutputBuildWrite($aFunctions[$i][0] & " wrong Include line at " & $aTxt[7] & @CRLF)
			EndIf
		EndIf
	Next
EndFunc   ;==>Update_TxtInclude

#EndRegion Update_TxtInclude()

#Region Verify_include()

Func Verify_include($sTxtFile, ByRef $aFunctions, $sFile)
	Local $sFunction = StringTrimRight($sFile, 4)
	For $i = 0 To UBound($aFunctions) - 1
		If $aFunctions[$i][0] <> $sFunction Then ContinueLoop
		If StringInStr($sTxtFile, "#include <" & $aFunctions[$i][1] & ">") Then
			$aFunctions[$i][1] = ""
			Return True
		EndIf
	Next
	Return False
EndFunc   ;==>Verify_include

#EndRegion Verify_include()

#Region GetCurrentFunctions()

Func GetCurrentFunctions($sIncludeFile, $sFile)
	Local $aFiles, $aFilesInclude
	Local $j, $i = StringInStr($sIncludeFile, "; #CURRENT# =")
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 2, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)

		Dim $aFilesInclude[UBound($aFiles)][2]

		UpdateIncludeFile($aFilesInclude, $aFiles, $sFile)

	EndIf

	Return $aFilesInclude
EndFunc   ;==>GetCurrentFunctions

#EndRegion GetCurrentFunctions()

#Region UpdateIncludeFile()

Func UpdateIncludeFile(ByRef $aFilesInclude, ByRef $aFiles, $sFile)
	If Not IsArray($aFiles) Then Return
	Local $k = 0, $vRange = "", $sCurInclude = $sFile, $sFunctionName
	For $i = 0 To UBound($aFiles) - 1
		$sFunctionName = $aFiles[$i]
		If StringLeft($sFunctionName, 6) = "Doc in" Then
			$vRange &= $k & ";"
			$sCurInclude = StringTrimLeft($sFunctionName, 7) & ".au3"
		Else
			$aFilesInclude[$i][0] = $sFunctionName
			$aFilesInclude[$i][1] = $sCurInclude
		EndIf
		$k += 1
	Next
	If $vRange Then
		$vRange = StringTrimRight($vRange, 1)
		_ArrayDelete($aFilesInclude, $vRange)
	EndIf
EndFunc   ;==>UpdateIncludeFile

#EndRegion UpdateIncludeFile()

#EndRegion Primary Functions
